class LodException(Exception):
    pass
    
class CastException(Exception):
    pass
    
class LodXYVException(Exception):
    pass
    
class LodYMVException(Exception):
    pass
    
class LolException(Exception):
    pass

class CurrencyOperationsException(Exception):
    pass
